#!/usr/bin/perl -w

# uvscan virus DAT file updater written by
# Michael Matsumura (michael+uvscan@limit.org)
# Modified and bug fixed for eng.cam.ac.uk by ceb4 and ecb39
# Version 1.2
# 
# Net::FTP is required for operation

use strict;

# Define global variables
my ($ftp, @dirlist, $arraywalk, $serverver, $localfile, @files, $file);

# Set the directory uvscan is located/installed in. 
my $uvscan_directory = "/usr/local/uvscan";

# Set to the temporary directory to download the DAT archive.
my $tempdir = "/usr/local/uvscan/dat";

my ($emailaddress, $ftpserver, $ftpdir);
# Set to email address for anonymous FTP login
my $configfile = "$uvscan_directory/script/config";
open (CONFIG, $configfile)
	or die "Unable to read $configfile: $!\n";
while (<CONFIG>) {
	chomp;
	$emailaddress = $1 if /^Email: (.*)/;
	$ftpserver = $1 if /^FTPServer: (.*)/;
	$ftpdir = $1 if /^FTPDir: (.*)/;
}

die "Config incomplete" unless $emailaddress && $ftpserver && $ftpdir;
my $highestserverver = 0;

# Get the local uvscan datfile version 
my $localver = &checkuvscanver;
print "Currently installed version: ".$localver."\n";

# Create FTP connection 
use Net::FTP;
$ftp = Net::FTP->new($ftpserver, Debug => 0, Passive => 1);

# Login 
$ftp->login("anonymous",$emailaddress) || die "login error\n";
$ftp->cwd($ftpdir) || die "ftp: could not cd to $ftpdir\n";

# Find highest version
@dirlist = $ftp->ls('dat*.zip');
unless(@dirlist) { die "No files in directory $ftpdir\n"; }
#print "Dir: @dirlist\n";
# work out which is latest version
foreach $arraywalk (@dirlist)
{   if ($arraywalk =~ /dat-([0-9]+)\.zip/i)
    {   $serverver = $1;
        $highestserverver = $serverver if ($serverver > $highestserverver);
    }
}

print "Latest Version on $ftpserver: ".$highestserverver."\n";

# If highest version is more recent download and install
if ($highestserverver > $localver) {
    print "Updating virus data files...\n";
    $serverver = $highestserverver;
    my $dat = "dat-$serverver.zip";

    # Create and then change the working directory to $tempdir
    if (!(-d $tempdir)) {
	mkdir($tempdir, 0700) or die("ERROR: Couldn't make temporary directory: $tempdir");
    }
    chdir $tempdir or die("ERROR: Couldn't change directory to tempdir: $tempdir");

    # Download the DAT file! 
    $ftp->binary();
    $localfile = $ftp->get($dat) || die "Couldn't download $dat\n";
    print "Download complete...updating now\n";

    # unzip the files
    my $error = `unzip $dat`;

    # move new dats into place and delete other files
    @files = `ls`;
    #print "Unzipped files: \n@files\n";
    foreach $file (@files) {
	# A line break is at the end of each $file...
	# chomp that off 
	chomp($file);

	# give all non-hidden files sensible permissions (often too permissive)
	if ($file !~ /^\..*/) {
	    chmod(0644,$file);
	}
	
	# We just move the .dat files to the uvscan_directory
	if ($file =~ /.*\.dat/) {
	    my $movestring = "mv $file ".$uvscan_directory."/".lc($file);
	    # and make sure they are lowercase.
	    #print " ".$movestring."\n";
	    system($movestring);
	}
	# delete non .dat files (except the .zip)
	else {
	    unless ($file =~ /\.zip/) {
		unlink($file) or die("ERROR: Couldn't delete file: $file");
	    }
	}
    }

    # Make sure that the installation worked,
    # by checking if the virus scanner reports
    # the same data file version as the one we downloaded. 
    if (&checkuvscanver eq $serverver)
    {   print "Installation successful\n";
    } else
    {   print "Error in installation, please install manually\n";
    }

    # Cleanup... 
    print "Cleaning up\n";
    # Remove downloaded DAT archive 
    unlink($dat) or die("ERROR: Couldn't delete DAT archive: $dat");
} 
elsif ($highestserverver == $localver) {
    print "DAT files are the same... no need to update\n";
}
else {
    print "No more recent DAT files found\n";
}

$ftp->quit;

# uvscan --version reports...
# "Virus data file 4229 created Oct 16 2002"
# &checkuvscanver returns the version
# of the data files.
sub checkuvscanver {
    if (`$uvscan_directory/uvscan --version` =~ /Virus data file v([0-9]+) created/) {
	return $1;
    } 
}
